;-*- mode: Scheme; tab-width: 8; fill-column: 70; -*- 
;$Id: control.scm,v 1.2 2005/07/20 12:33:43 inger Exp $ 

(use-translation _ "alterator-control")

(include-as-is "/dialog/functions")

(on-apply (change-status))

(on-acc-leave (if (next-button enabled) 'need-confirm))

((global 'disable-next))

(define (on-load-error reason) '())
(define (on-update-error reason)
  (username text (string-append "Error:" reason)))

(define *marked-facility* #f)

(define (clear-dirty)
  (set! *marked-facility* #f)
  (next-button enabled #f))

(define (mark-as-dirty)
  (set! *marked-facility* (selected-facility))
  (next-button enabled #t))

(define (ask-and-apply)
  (if (and (next-button enabled)
	   (run-dialog '("/control/apply_question")))
      (begin (change-status)
	     (next-button enabled #f))
      #t))

(define (get-facilities)
  (woo-read-names "/control"
		  (woo-list "/control")))

(define (get-facility-info fc)
  (woo-first-command
   (woo-read (string-append "/control/" fc))))

(define *facilities* (woo-catch
		      get-facilities
		      on-load-error))

(define (selected-facility) (facilities text))
(define (selected-status) (status text))

(define (update-status-info stat status-str)
  (let ((status-list (string-splitting status-str #\space)))
    (status (items status-list))
    (and (not (string= "unknown" stat)) (status current (string-list-index stat status-list)))))

(define facility-info '())

(define (update-description-info woo-param)
  (let ((descr (woo-get-option facility-info woo-param)))
    (description text (or (and (string=? descr "") (_ "This status has no description yet.")) descr))))

(define (update-facility-info fc)
  (set! facility-info (get-facility-info fc))
  (facility text (facilities text))
  (cur-status text (woo-get-option facility-info 'status))
  (update-status-info 
   (woo-get-option facility-info 'status) 
   (woo-get-option facility-info 'list))
  (update-description-info (string->symbol (selected-status))))

(define (update-status-desc stat)
  (update-description-info (string->symbol (selected-status))))

(define (change-status)
  (and 
   *marked-facility*
   (not (string=? (selected-status) (woo-get-option facility-info 'status))) 
   
   (woo-catch
    (lambda ()
      (and 
       (woo-write (string-append "/control/" *marked-facility*) 'status (selected-status))
       (update-facility-info *marked-facility*)
       ))
    (lambda(reason)
      (run-dialog `("/error" reason ,reason))))))

(hbox
 spacing 1
 (id 'facilities
     (listbox (items *facilities*)
					; sizepolicy '(fixed . expanding)
	      (relative-height 50 page)
	      (relative-width 30 page)
	      (on-select
	       (ask-and-apply)
	       (clear-dirty)
	       (woo-catch
		(lambda() (update-facility-info (selected-facility)))
		on-update-error))))
 (vbox
  spacing 1
  sizepolicy 'fixed
  (hbox
   (label (_ "Name:"))
   (id 'facility (label "" height 3)))
  (hbox
   (label (_ "Current status:"))
   (id 'cur-status (label "" height 3)))
  (hbox
   (label (_ "Status:"))
   (id 'status (combobox (items '())
			 (on-change
			  (mark-as-dirty)
			  (woo-catch
			   (lambda() (update-status-desc (selected-status)))
			   on-update-error)))))
  (id 'description (textbox "" readonly "yes"))))
